@@ -23,6 +23,7 @@
 
 #include "gkd-dbus-util.h"
 #include "gkd-secret-dispatch.h"
+#include "gkd-secret-introspect.h"
 #include "gkd-secret-objects.h"
 #include "gkd-secret-secret.h"
 #include "gkd-secret-session.h"
@@ -326,7 +327,7 @@ gkd_secret_unlock_real_dispatch_message (GkdSecretDispatch *base, DBusMessage *m
 		reply = prompt_method_dismiss (self, message);
 
 	else if (dbus_message_has_interface (message, DBUS_INTERFACE_INTROSPECTABLE))
-		return gkd_dbus_introspect_handle (message, "prompt");
+		return gkd_dbus_introspect_handle (message, gkd_secret_introspect_prompt);
 
 	return reply;
 }
